class StoriesController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user!, :only => [:new, :create, :edit, :update, :destroy]
  else
    before_filter :authenticate_user!, :only => [:index, :show, :comic, :new, :create, :edit, :update, :destroy]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def show
    @story = Story.show(params[:id], @author)

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @story.story_as_json(@author) }
    end
  end
  
  def comic
    @comic = Comic.show(params[:id], @author)
    cnt = Story.count(:conditions => ['comic_id = ?', @comic.id]).to_i
    @offset = Story.offset cnt, params[:offset]
    @panel_count = Story.panel_count cnt, params[:count]
    @stories = Story.play_list(@comic, @author, @offset, @panel_count)
    respond_to do |format|
      format.html # index.html.erb
      format.json {render text: Story.list_as_json_text(@stories, @author)}
      format.jsonp {
        render :json => "callback(" + @stories.to_json_list + ");"
      }
    end
  end
  
  def list
    @stories = Story.all

    respond_to do |format|
      format.html { render layout: 'system' }# index.html.erb
      format.json { render json: @stories }
    end
  end

  def browse
    @story = Story.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' } # show.html.erb
      format.json { render json: @story }
    end
  end
  
  # GET /stories/new
  # GET /stories/new.js
  # GET /stories/new.json
  def new
    @story = Story.new 
    @story.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @story.story_as_json(@author) }
    end
  end

  # GET /stories/1/edit
  # GET /stories/1.js/edit
  def edit
    @story = Story.edit(params[:id], @author)
    respond_to do |format|
      format.html 
      format.js
    end
  end

  # POST /stories
  # POST /stories.json
  def create
    @story = Story.new 
    @story.supply_default
    @story.attributes = params[:story]
    @story.overwrite @author
    @comic = Comic.edit(@story.comic_id, @author)
    @panel = Panel.show(@story.panel_id, @author)
    
    respond_to do |format|
      if @story.store
        format.html { redirect_to action: :comic, id: @story.comic_id }
        format.json { render json: @story.story_as_json(@author) }
      else
        format.html { render action: "new" }
        format.json { render json: @story.errors, status: :unprocessable_entity }
      end
    end
  end
  
  # PUT /stories/1
  # PUT /stories/1.json
  def update
    @story = Story.edit(params[:id], @author)
    ot = @story.t
    @story.attributes = params[:story]
    @story.overwrite @author
    respond_to do |format|
      if @story.store ot
        format.html { redirect_to action: :comic, id: @story.comic_id }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @story.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /stories/1
  # DELETE /stories/1.js
  # DELETE /stories/1.json
  def destroy
    @story = Story.edit(params[:id], @author)
    Story.transaction do
      @story.destroy_and_shorten
    end
    respond_to do |format|
      format.html { redirect_to story_path(@story.comic) }
      format.json { head :ok }
    end
  end
end
